#include <math.h>
#include <stdint.h>
#include <stdlib.h>

/*
static char encoding_table[] = {'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H',
                                'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P',
                                'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X',
                                'Y', 'Z', 'a', 'b', 'c', 'd', 'e', 'f',
                                'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n',
                                'o', 'p', 'q', 'r', 's', 't', 'u', 'v',
                                'w', 'x', 'y', 'z', '0', '1', '2', '3',
                                '4', '5', '6', '7', '8', '9', '+', '/'};
static char *decoding_table = NULL;
static int mod_table[] = {0, 2, 1};


char *base64_encode(const char *data,
                    size_t input_length,
                    size_t *output_length) {

    printf("%04x\n", *(int *)(data + 3));

    *output_length = (size_t) (4.0 * ceil((double) input_length / 3.0));

    char *encoded_data = malloc(*output_length + 1);
    if (encoded_data == NULL) return NULL;

    for (int i = 0, j = 0; i < input_length;) {

        uint32_t octet_a = i < input_length ? data[i++] : 0;
        uint32_t octet_b = i < input_length ? data[i++] : 0;
        uint32_t octet_c = i < input_length ? data[i++] : 0;

        uint32_t triple = (octet_a << 0x10) + (octet_b << 0x08) + octet_c;

        encoded_data[j++] = encoding_table[(triple >> (3 * 6)) & 0x3F];
        encoded_data[j++] = encoding_table[(triple >> (2 * 6)) & 0x3F];
        encoded_data[j++] = encoding_table[(triple >> (1 * 6)) & 0x3F];
        encoded_data[j++] = encoding_table[(triple >> (0 * 6)) & 0x3F];
    }

    for (int i = 0; i < mod_table[input_length % 3]; i++)
        encoded_data[*output_length - 1 - i] = '=';

    encoded_data[*output_length] = 0;

    return encoded_data;
}

void build_decoding_table();

char *base64_decode(const char *data,
                    size_t input_length,
                    size_t *output_length) {

    if (decoding_table == NULL) build_decoding_table();

    if (input_length % 4 != 0) return NULL;

    *output_length = input_length / 4 * 3;
    if (data[input_length - 1] == '=') (*output_length)--;
    if (data[input_length - 2] == '=') (*output_length)--;

    char *decoded_data = malloc(*output_length);
    if (decoded_data == NULL) return NULL;

    for (int i = 0, j = 0; i < input_length;) {

        uint32_t sextet_a = data[i] == '=' ? 0 & i++ : decoding_table[data[i++]];
        uint32_t sextet_b = data[i] == '=' ? 0 & i++ : decoding_table[data[i++]];
        uint32_t sextet_c = data[i] == '=' ? 0 & i++ : decoding_table[data[i++]];
        uint32_t sextet_d = data[i] == '=' ? 0 & i++ : decoding_table[data[i++]];

        uint32_t triple = (sextet_a << 3 * 6)
                        + (sextet_b << 2 * 6)
                        + (sextet_c << 1 * 6)
                        + (sextet_d << 0 * 6);

        if (j < *output_length) decoded_data[j++] = (triple >> 2 * 8) & 0xFF;
        if (j < *output_length) decoded_data[j++] = (triple >> 1 * 8) & 0xFF;
        if (j < *output_length) decoded_data[j++] = (triple >> 0 * 8) & 0xFF;
    }

    return decoded_data;
}


void build_decoding_table() {

    decoding_table = malloc(256);

    for (int i = 0; i < 0x40; i++)
        decoding_table[encoding_table[i]] = i;
}


void base64_cleanup() {
    free(decoding_table);
}

*/

static char *b64_lut = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
 
void bin_to_b64(char *out, char* in, unsigned int in_length)
{
    unsigned int complete_quads = in_length / 3;
    unsigned int additional = in_length - complete_quads * 3;
 
    for (unsigned int i = 0; i < complete_quads; i++)
    {
        char a, b, c;
        a = in[i * 3]; b = in[i * 3 + 1]; c = in[i * 3 + 2];
 
        out[i * 4    ] = b64_lut[ (a & 252) >> 2                   ];
        out[i * 4 + 1] = b64_lut[((a & 3) << 4 ) | ((b & 240) >> 4)];
        out[i * 4 + 2] = b64_lut[((b & 15) << 2) | ((c & 192) >> 6)];
        out[i * 4 + 3] = b64_lut[  c & 63                          ];
    }
 
    unsigned int final_length = complete_quads * 4;
 
    switch (additional)
    {
        case 1:
        {
            char a = in[complete_quads * 3];
            out[complete_quads * 4    ] = b64_lut[(a & 252) >> 2];
            out[complete_quads * 4 + 1] = b64_lut[(a & 3) << 4];
            out[complete_quads * 4 + 2] = '=';
            out[complete_quads * 4 + 3] = '=';
            final_length += 4;
            break;
        }
        case 2:
        {
            char a, b;
            a = in[complete_quads * 3]; b = in[complete_quads * 3 + 1];
            out[complete_quads * 4    ] = b64_lut[ (a & 252) >> 2                  ];
            out[complete_quads * 4 + 1] = b64_lut[((a & 3) << 4) | ((b & 240) >> 4)];
            out[complete_quads * 4 + 2] = b64_lut[((b & 15) << 2)                  ];
            out[complete_quads * 4 + 3] = '=';
            final_length += 4;
            break;
        }
    }
    out[final_length] = 0;
}
 
